@@ -1,3 +1,5 @@
+"""Geo lookup provided courtesy of freegeoip.net"""
+
 import argparse
 import requests
 import socket
@@ -8,6 +10,7 @@ import peer
 import time
 import os
 import random
+import json
 from string import ascii_letters, digits
 from listener import Listener
 from switchboard import Switchboard
@@ -18,6 +21,13 @@ DEFAULT_PORT = 55308
 DEFAULT_DIR = '~/Desktop'
 
 
+def connect_vis(address):
+    ip, port = address.split(':')
+    vis_addr_tuple = (ip, int(port))
+    mysock = socket.socket()
+    mysock.connect(vis_addr_tuple)
+    return mysock
+
 def get_path(file_path):
     if file_path[0] == '.':
         return os.path.abspath(file_path)
@@ -61,7 +71,7 @@ class VisListener(Listener):
 class Torrent(object):
 
     def __init__(self, torrent_path, directory='', port=55308,
-                 download_all=False):
+                 download_all=False, visualizer=None):
         torrent_dict = tparser.bdecode_file(torrent_path)
         self.torrent_dict = torrent_dict
         self.peer_dict = {}
@@ -77,6 +87,9 @@ class Torrent(object):
         self.reactor = reactor.Reactor()
         self.reactor.add_listeners([PeerListener(torrent=self, port=7000),
                                     VisListener(torrent=self, port=8035)])
+
+        # Try to connect to visualization server
+        vis_socket = connect_vis(visualizer) if visualizer else None
         if directory:
             os.chdir(directory)
         if 'files' in self.torrent_dict['info']:
@@ -105,7 +118,7 @@ class Torrent(object):
                                        file_list=file_list, piece_length=
                                        self.piece_length, num_pieces=
                                        self.num_pieces, multifile=multifile,
-                                       download_all=download_all)
+                                       download_all=download_all, vis_socket=vis_socket)
 
     @property
     def piece_length(self):
@@ -251,8 +264,10 @@ class Torrent(object):
         Creates a new peer object for a nvalid socket and adds it to reactor's
         listen list
         '''
-
-        tpeer = peer.Peer(sock, self.reactor, self)
+        location_json = requests.request("GET", "http://freegeoip.net/json/"
+                                         + sock.getpeername()[0]).content
+        location = json.loads(location_json)
+        tpeer = peer.Peer(sock, self.reactor, self, location)
         self.peer_dict[sock] = tpeer
         self.reactor.select_list.append(tpeer)
 
@@ -289,17 +304,31 @@ def main():
     argparser.add_argument('-a', '--all', action='store_true',
                            help=('Set flag to download all files in torrent'),
                            default=False)
+    argparser.add_argument('-v', '--visualizer',
+                           help=('Colon-separated address and port for running'
+                                 'visualizer. \ne.g., "127.0.0.1:8000'),
+                           default=None, type=open_socket)
     args = argparser.parse_args()  # Getting path from command line
     torrent_path = get_path(args.torrent_path)
     directory = get_path(args.directory)
+    visualizer = args.visualizer    
     port = args.port
     download_all = args.all
     mytorrent = Torrent(torrent_path, directory=directory, port=port,
-                        download_all=download_all)
+                        download_all=download_all, visualizer=visualizer)
     with mytorrent:
         mytorrent.tracker_request()
         mytorrent.handshake_peers()
         mytorrent.reactor.event_loop()
 
+def open_socket(url):
+    split_url = url.split(':')
+    split_url[1] = int(split_url[1])
+    split_url_tuple = tuple(split_url)
+    mysock = socket.create_connection(split_url_tuple, 2)
+    mysock.close()
+    return url
+
+
 if __name__ == '__main__':
     main()
